@@ -3,7 +3,7 @@
 #
 # kluster/kox point process models
 #
-# $Revision: 1.195 $ $Date: 2021/11/22 00:39:32 $
+# $Revision: 1.209 $ $Date: 2022/04/01 03:05:01 $
 #
 
 
@@ -105,7 +105,7 @@ kppm.ppp <- kppm.quad <-
                       statargs=statargs,
                       rmax = rmax)
   Xenv <- list2env(as.list(covariates), parent=parent.frame())
-  X <- eval(substitute(X), envir=Xenv, enclos=baseenv())
+  X <- eval(substitute(X), envir=Xenv, enclos=parent.frame())
   isquad <- is.quad(X)
   if(!is.ppp(X) && !isquad)
     stop("X should be a point pattern (ppp) or quadrature scheme (quad)")
@@ -128,7 +128,8 @@ kppm.ppp <- kppm.quad <-
             covfunargs=covfunargs, use.gam=use.gam, nd=nd, eps=eps)
   XX <- if(isquad) X$data else X
   
-  ## set default weight function
+           
+  ## default weight function
   if(is.null(weightfun))
     switch(method,
            adapcl = {
@@ -151,15 +152,18 @@ kppm.ppp <- kppm.quad <-
                              control=control, stabilize=stabilize,
                              statistic=statistic,
                              statargs=statargs, rmax=rmax,
-                             algorithm=algorithm, ...),
+                             algorithm=algorithm,
+                             ...),
          clik2   = kppmComLik(X=XX, Xname=Xname, po=po, clusters=clusters,
                              control=control, stabilize=stabilize,
                              weightfun=weightfun, 
-                             rmax=rmax, algorithm=algorithm, ...),
+                             rmax=rmax, algorithm=algorithm,
+                             ...),
          palm   = kppmPalmLik(X=XX, Xname=Xname, po=po, clusters=clusters,
                              control=control, stabilize=stabilize,
                              weightfun=weightfun, 
-                             rmax=rmax, algorithm=algorithm, ...),
+                             rmax=rmax, algorithm=algorithm,
+                             ...),
          adapcl   = kppmCLadap(X=XX, Xname=Xname, po=po, clusters=clusters,
                              control=control, epsilon=epsilon, 
                              weightfun=weightfun, rmax=rmax,
@@ -186,8 +190,13 @@ kppm.ppp <- kppm.quad <-
   return(out)
 }
 
+## >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+##                M  i  n  i  m  u  m       C  o  n  t  r  a  s  t
+## <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+
 kppmMinCon <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, statistic, statargs,
-                       algorithm="Nelder-Mead", DPP=NULL, ...) {
+                       algorithm="Nelder-Mead", DPP=NULL, ...,
+                       pspace=NULL) {
   # Minimum contrast fit
   stationary <- is.stationary(po)
   # compute intensity
@@ -209,20 +218,21 @@ kppmMinCon <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, s
   mcfit <- clusterfit(X, clusters, lambda = lambda,
                       dataname = Xname, control = control,  stabilize=stabilize,
                       statistic = statistic, statargs = statargs,
-                      algorithm=algorithm, ...)
+                      algorithm=algorithm, pspace=pspace, ...)
   fitinfo <- attr(mcfit, "info")
   attr(mcfit, "info") <- NULL
   # all info that depends on the fitting method:
-  Fit <- list(method    = "mincon",
-              statistic = statistic,
-              Stat      = fitinfo$Stat,
-              StatFun   = fitinfo$StatFun,
-              StatName  = fitinfo$StatName,
-              FitFun    = fitinfo$FitFun,
-              statargs  = statargs,
-              pspace    = fitinfo$pspace,
-              mcfit     = mcfit,
-              maxlogcl  = NULL)
+  Fit <- list(method       = "mincon",
+              statistic    = statistic,
+              Stat         = fitinfo$Stat,
+              StatFun      = fitinfo$StatFun,
+              StatName     = fitinfo$StatName,
+              FitFun       = fitinfo$FitFun,
+              statargs     = statargs,
+              pspace.given = pspace,
+              pspace.used  = fitinfo$pspace.used, 
+              mcfit        = mcfit,
+              maxlogcl     = NULL)
   # results
   if(!is.null(DPP)){
     clusters <- update(clusters, as.list(mcfit$par))
@@ -345,7 +355,7 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
           if(isDPP)
               stop("No rule for starting parameters in this case. Please set ",
                    sQuote("startpar"), " explicitly.")
-          startpar <- info$checkpar(startpar, old=FALSE)
+          startpar <- info$checkpar(startpar, native=FALSE)
           startpar[["scale"]] <- mean(range(Stat[[fvnames(Stat, ".x")]]))
       }
   } else{
@@ -371,9 +381,9 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
     algorithm <- alg$algorithm
   }
 
-  dots <- info$resolvedots(...)
+  dots <- info$resolveshape(...)
   #' determine initial values of parameters
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   #' code to compute the theoretical summary function of the model
   theoret <- info[[statistic]]
   #' explanatory text
@@ -393,14 +403,14 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
                                       modelname=info$modelname),
                                   margs=dots$margs,
                                   model=dots$model,
-                                  funaux=info$funaux,
-                                  pspace=pspace),
+                                  pspace=pspace), ## As modified above
                              list(...)
                              )
 
   if(isDPP && algorithm=="Brent" && changealgorithm)
     mcargs <- resolve.defaults(mcargs, list(lower=alg$lower, upper=alg$upper))
 
+  
   ## .............. FIT .......................
   if(verbose) splat("Starting minimum contrast fit")
   mcfit <- do.call(mincontrast, mcargs)
@@ -443,28 +453,33 @@ clusterfit <- function(X, clusters, lambda = NULL, startpar = NULL,
   }
   ## Parameter values (new format)
   mcfit$mu <- mu
-  mcfit$clustpar <- info$checkpar(mcfit$par, old=FALSE)
-  mcfit$clustargs <- info$checkclustargs(dots$margs, old=FALSE)
+  mcfit$clustpar <- info$checkpar(mcfit$par, native=FALSE)
+  mcfit$clustargs <- info$outputshape(dots$margs)
 
   ## The old fit fun that would have been used (DO WE NEED THIS?)
   FitFun <- paste0(tolower(clusters), ".est", statistic)
 
-  extra <- list(FitFun    = FitFun,
-                Stat      = Stat,
-                StatFun   = StatFun,
-                StatName  = StatName,
-                modelname  = info$modelabbrev,
-                isPCP      = isPCP,
-                lambda     = lambda,
-                pspace     = pspace)
+  extra <- list(FitFun       = FitFun,
+                Stat         = Stat,
+                StatFun      = StatFun,
+                StatName     = StatName,
+                modelname    = info$modelabbrev,
+                isPCP        = isPCP,
+                lambda       = lambda,
+                pspace.used  = pspace) # Modified from call to 'clusterfit'
   attr(mcfit, "info") <- extra
   if(verbose) splat("Returning from clusterfit")
   return(mcfit)
 }
 
+## >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+##                C  o  m  p  o  s  i  t  e    L  i  k  e  l  i  h  o  o  d
+## <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 
-kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, weightfun, rmax,
-                       algorithm="Nelder-Mead", DPP=NULL, ..., pspace=NULL) {
+kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
+                       weightfun, rmax, algorithm="Nelder-Mead",
+                       DPP=NULL, ..., 
+                       pspace=NULL) {
   W <- as.owin(X)
   if(is.null(rmax))
     rmax <- rmax.rule("K", W, intensity(X))
@@ -528,23 +543,21 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
   g <- g[with(g, .x) <= rmax,]
   # get pair correlation function (etc) for model
   info <- spatstatClusterModelInfo(clusters)
-  pcfun      <- info$pcf
-  funaux     <- info$funaux
-  selfstart  <- info$selfstart
-  isPCP      <- info$isPCP
-  parhandler <- info$parhandler
-  modelname  <- info$modelname
+  pcfun        <- info$pcf
+  selfstart    <- info$selfstart
+  isPCP        <- info$isPCP
+  resolveshape <- info$resolveshape
+  modelname    <- info$modelname
   # Assemble information required for computing pair correlation
-  pcfunargs <- list(funaux=funaux)
-  if(is.function(parhandler)) {
-    # Additional parameters of cluster model are required.
+  if(is.function(resolveshape)) {
+    # Additional 'shape' parameters of cluster model are required.
     # These may be given as individual arguments,
     # or in a list called 'covmodel'
     clustargs <- if("covmodel" %in% names(otherargs))
                  otherargs[["covmodel"]] else otherargs
-    clargs <- do.call(parhandler, clustargs)
-    pcfunargs <- append(clargs, pcfunargs)
-  } else clargs <- NULL
+    shapemodel <- do.call(resolveshape, clustargs)$covmodel
+  } else shapemodel <- NULL
+  pcfunargs <- shapemodel
   # determine starting parameter values
   startpar <- selfstart(X)
   
@@ -576,7 +589,7 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
       },
       enclos=objargs$envir)
     }
-    ## determine a suitable large number to replace Inf
+    ## Determine a suitable large number to replace Inf
     objargs$BIGVALUE <- bigvaluerule(obj, objargs, startpar)
   } else {
     # create local function to evaluate  pair correlation(d) * weight(d)
@@ -608,7 +621,7 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
       },
       enclos=objargs$envir)
     }
-    ## determine a suitable large number to replace Inf
+    ## Determine a suitable large number to replace Inf
     objargs$BIGVALUE <- bigvaluerule(obj, objargs, startpar)
   }
 
@@ -663,14 +676,15 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
   ## Finish in DPP case
   if(!is.null(DPP)){
     ## all info that depends on the fitting method:
-    Fit <- list(method    = "clik2",
-                clfit     = opt,
-                weightfun = weightfun,
-                rmax      = rmax,
-                objfun    = obj,
-                objargs   = objargs,
-                maxlogcl  = opt$value,
-                pspace    = pspace)
+    Fit <- list(method       = "clik2",
+                clfit        = opt,
+                weightfun    = weightfun,
+                rmax         = rmax,
+                objfun       = obj,
+                objargs      = objargs,
+                maxlogcl     = opt$value,
+                pspace.given = pspace,
+                pspace.used  = pspace)
     # pack up
     clusters <- update(clusters, as.list(opt$par))
     result <- list(Xname      = Xname,
@@ -700,14 +714,15 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
           eval.im(log(lambda) - sigma2/2)    
   }
   # all info that depends on the fitting method:
-  Fit <- list(method    = "clik2",
-              clfit     = opt,
-              weightfun = weightfun,
-              rmax      = rmax,
-              objfun    = obj,
-              objargs   = objargs,
-              maxlogcl  = opt$value,
-              pspace    = pspace)
+  Fit <- list(method       = "clik2",
+              clfit        = opt,
+              weightfun    = weightfun,
+              rmax         = rmax,
+              objfun       = obj,
+              objargs      = objargs,
+              maxlogcl     = opt$value,
+              pspace.given = pspace,
+              pspace.used  = pspace)
   # pack up
   result <- list(Xname      = Xname,
                  X          = X,
@@ -719,10 +734,10 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
                  lambda     = lambda,
                  mu         = mu,
                  par        = optpar,
-                 clustpar   = info$checkpar(par=optpar, old=FALSE),
-                 clustargs  = info$checkclustargs(clargs$margs, old=FALSE), #clargs$margs,
+                 clustpar   = info$checkpar(par=optpar, native=FALSE),
+                 clustargs  = info$outputshape(shapemodel$margs),
                  modelpar   = modelpar,
-                 covmodel   = clargs,
+                 covmodel   = shapemodel,
                  Fit        = Fit)
   
 
@@ -730,8 +745,15 @@ kppmComLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, w
 }
 
 
+## >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+##                P  a  l  m     L  i  k  e  l  i  h  o  o  d              
+## <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+
+
 kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE, weightfun, rmax,
-                        algorithm="Nelder-Mead", DPP=NULL, ..., pspace=NULL) {
+                        algorithm="Nelder-Mead", DPP=NULL, ...,
+                        
+                        pspace=NULL) {
   W <- as.owin(X)
   if(is.null(rmax))
     rmax <- rmax.rule("K", W, intensity(X))
@@ -796,23 +818,21 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
   g <- g[with(g, .x) <= rmax,]
   # get pair correlation function (etc) for model
   info <- spatstatClusterModelInfo(clusters)
-  pcfun      <- info$pcf
-  funaux     <- info$funaux
-  selfstart  <- info$selfstart
-  isPCP      <- info$isPCP
-  parhandler <- info$parhandler
-  modelname  <- info$modelname
+  pcfun        <- info$pcf
+  selfstart    <- info$selfstart
+  isPCP        <- info$isPCP
+  resolveshape <- info$resolveshape
+  modelname    <- info$modelname
   # Assemble information required for computing pair correlation
-  pcfunargs <- list(funaux=funaux)
-  if(is.function(parhandler)) {
-    # Additional parameters of cluster model are required.
+  if(is.function(resolveshape)) {
+    # Additional 'shape' parameters of cluster model are required.
     # These may be given as individual arguments,
     # or in a list called 'covmodel'
     clustargs <- if("covmodel" %in% names(otherargs))
                  otherargs[["covmodel"]] else otherargs
-    clargs <- do.call(parhandler, clustargs)
-    pcfunargs <- append(clargs, pcfunargs)
-  } else clargs <- NULL
+    shapemodel <- do.call(resolveshape, clustargs)$covmodel
+  } else shapemodel <- NULL
+  pcfunargs <- shapemodel
   # determine starting parameter values
   startpar <- selfstart(X)
   
@@ -845,7 +865,7 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
       },
       enclos=objargs$envir)
     }
-    ## determine a suitable large number to replace Inf
+    ## Determine a suitable large number to replace Inf
     objargs$BIGVALUE <- bigvaluerule(obj, objargs, startpar)
   } else {
     # create local function to evaluate  pair correlation(d) * weight(d)
@@ -878,7 +898,7 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
       },
       enclos=objargs$envir)
     }
-    ## determine a suitable large number to replace Inf
+    ## Determine a suitable large number to replace Inf
     objargs$BIGVALUE <- bigvaluerule(obj, objargs, startpar)
   }
 
@@ -934,14 +954,15 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
   ## Finish in DPP case
   if(!is.null(DPP)){
     ## all info that depends on the fitting method:
-    Fit <- list(method    = "palm",
-                clfit     = opt,
-                weightfun = weightfun,
-                rmax      = rmax,
-                objfun    = obj,
-                objargs   = objargs,
-                maxlogcl  = opt$value,
-                pspace    = pspace)
+    Fit <- list(method       = "palm",
+                clfit        = opt,
+                weightfun    = weightfun,
+                rmax         = rmax,
+                objfun       = obj,
+                objargs      = objargs,
+                maxlogcl     = opt$value,
+                pspace.given = pspace,
+                pspace.used  = pspace)
     # pack up
     clusters <- update(clusters, as.list(optpar))
     result <- list(Xname      = Xname,
@@ -970,14 +991,15 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
           eval.im(log(lambda) - sigma2/2)    
   }
   # all info that depends on the fitting method:
-  Fit <- list(method    = "palm",
-              clfit     = opt,
-              weightfun = weightfun,
-              rmax      = rmax,
-              objfun    = obj,
-              objargs   = objargs,
-              maxlogcl  = opt$value,
-              pspace    = pspace)
+  Fit <- list(method       = "palm",
+              clfit        = opt,
+              weightfun    = weightfun,
+              rmax         = rmax,
+              objfun       = obj,
+              objargs      = objargs,
+              maxlogcl     = opt$value,
+              pspace.given = pspace,
+              pspace.used  = pspace)
   # pack up
   result <- list(Xname      = Xname,
                  X          = X,
@@ -989,15 +1011,20 @@ kppmPalmLik <- function(X, Xname, po, clusters, control=list(), stabilize=TRUE,
                  lambda     = lambda,
                  mu         = mu,
                  par        = optpar,
-                 clustpar   = info$checkpar(par=optpar, old=FALSE),
-                 clustargs  = info$checkclustargs(clargs$margs, old=FALSE), #clargs$margs,
+                 clustpar   = info$checkpar(par=optpar, native=FALSE),
+                 clustargs  = info$outputshape(shapemodel$margs),
                  modelpar   = modelpar,
-                 covmodel   = clargs,
+                 covmodel   = shapemodel,
                  Fit        = Fit)
   return(result)
 }
 
 
+## >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
+##       A  d  a  p  t  i  v  e     C  o  m  p  o  s  i  t  e    L  i  k  e  l  i  h  o  o  d
+## <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+
+
 ## ........... contributed by Chiara Fend ...................
 ## needs nonlinear equation solver nleqslv
 
@@ -1058,31 +1085,28 @@ kppmCLadap <- function(X, Xname, po, clusters, control, weightfun,
   
   # get pair correlation function (etc) for model
   info <- spatstatClusterModelInfo(clusters)
-  pcfun      <- info$pcf
-  dpcfun     <- info$Dpcf
-  funaux     <- info$funaux
-  selfstart  <- info$selfstart
-  isPCP      <- info$isPCP
-  parhandler <- info$parhandler
-  modelname  <- info$modelname
+  pcfun        <- info$pcf
+  dpcfun       <- info$Dpcf
+  selfstart    <- info$selfstart
+  isPCP        <- info$isPCP
+  resolveshape <- info$resolveshape
+  modelname    <- info$modelname
   # Assemble information required for computing pair correlation
-  pcfunargs <- list(funaux=funaux)
-  if(is.function(parhandler)) {
+  if(is.function(resolveshape)) {
     # Additional parameters of cluster model are required.
     # These may be given as individual arguments,
     # or in a list called 'covmodel'
     clustargs <- if("covmodel" %in% names(otherargs))
       otherargs[["covmodel"]] else otherargs
-    clargs <- do.call(parhandler, clustargs)
-    pcfunargs <- append(clargs, pcfunargs)
-  } else clargs <- NULL
+    shapemodel <- do.call(resolveshape, clustargs)$covmodel
+  } else shapemodel <- NULL
+  pcfunargs <- shapemodel
   
   ## determine starting parameter values
   if(is.null(startpar)) {
     startpar <- selfstart(X)
   } else if(!isDPP){
-    checkpar  <- info$checkpar
-    startpar <- checkpar(startpar, old=TRUE)
+    startpar <- info$checkpar(startpar, native=TRUE)
   }
   ## optimization will be over the logarithms of the parameters
   startparLog <- log(startpar)
@@ -1219,10 +1243,10 @@ kppmCLadap <- function(X, Xname, po, clusters, control, weightfun,
                  lambda     = lambda,
                  mu         = mu,
                  par        = optpar,
-                 clustpar   = info$checkpar(par=optpar, old=FALSE),
-                 clustargs  = info$checkclustargs(clargs$margs, old=FALSE),
+                 clustpar   = info$checkpar(par=optpar, native=FALSE),
+                 clustargs  = info$outputshape(shapemodel$margs),
                  modelpar   = modelpar,
-                 covmodel   = clargs,
+                 covmodel   = shapemodel,
                  Fit        = Fit)
   return(result)
 }
@@ -1683,18 +1707,20 @@ update.kppm <- function(object, ..., evaluate=TRUE, envir=environment(terms(obje
   methodname <- as.character(thecall[[1L]])
   switch(methodname,
          kppm.formula = {
-	   # original call has X = [formula with lhs]
+	   ## original call has X = [formula with lhs]
 	   if(!is.null(Xexpr)) {
 	     lhs.of.formula(fmla) <- Xexpr
 	   } else if(is.null(lhs.of.formula(fmla))) {
 	     lhs.of.formula(fmla) <- as.name('.')
 	   }
-           oldformula <- as.formula(getCall(object)$X)
+           oldformula <- getCall(object)$X
+           oldformula <- eval(oldformula, callframe)
            thecall$X <- newformula(oldformula, fmla, callframe, envir)
          },
          {
-	   # original call has X = ppp and trend = [formula without lhs]
-           oldformula <- as.formula(getCall(object)$trend %orifnull% (~1))
+	   ## original call has X = ppp and trend = [formula without lhs]
+           oldformula <- getCall(object)$trend %orifnull% (~1)
+           oldformula <- eval(oldformula, callframe)
 	   fom <-  newformula(oldformula, fmla, callframe, envir)
 	   if(!is.null(Xexpr))
 	      lhs.of.formula(fom) <- Xexpr
@@ -1788,15 +1814,13 @@ Kpcf.kppm <- function(model, what=c("K", "pcf", "kernel")) {
          "cluster model")
   # Extract model parameters
   par <- model$par
-  # Extract auxiliary definitions (if applicable)
-  funaux <- tableentry$funaux
   # Extract covariance model (if applicable)
   cm <- model$covmodel
   model <- cm$model
   margs <- cm$margs
   #
   f <- function(r) as.numeric(fun(par=par, rvals=r,
-                                  funaux=funaux, model=model, margs=margs))
+                                  model=model, margs=margs))
   return(f)
 }
 
@@ -1899,3 +1923,20 @@ psib.kppm <- function(object) {
   return(p)
 }
 
+reach.kppm <- function(x, ..., epsilon) {
+  thresh <- if(missing(epsilon)) NULL else epsilon
+  if(x$isPCP) 
+    return(2 * clusterradius.kppm(x, ..., thresh=thresh))
+  ## use pair correlation
+  g <- pcfmodel(x)
+  ## find upper bound
+  if(is.null(thresh)) thresh <- 0.01
+  f <- function(r) { g(r) - 1 - thresh }
+  scal <- as.list(x$par)$scale %orifnull% 1
+  for(a in scal * 2^(0:10)) { if(f(a) < 0) break; }
+  if(f(a) > 0) return(Inf)
+  ## solve g(r) = 1 + epsilon
+  b <- uniroot(f, c(0, a))$root
+  return(b)
+}
+
