@@ -1,9 +1,9 @@
-#'
+c#'
 #'  mincontrast.R
 #'
 #'  Functions for estimation by minimum contrast
 #'
-#'  $Revision: 1.115 $ $Date: 2021/11/09 07:32:58 $
+#'  $Revision: 1.122 $ $Date: 2022/03/22 00:52:06 $
 #' 
 
 
@@ -84,6 +84,7 @@ mincontrast <- local({
                           pspace=NULL) {
     verifyclass(observed, "fv")
     action.bad.values <- match.arg(action.bad.values)
+
     
     stopifnot(is.function(theoretical))
     if(!any("par" %in% names(formals(theoretical))))
@@ -242,7 +243,8 @@ mincontrast <- local({
                    startpar = startpar,
                    objfun   = contrast.objective,
                    objargs  = objargs,
-                   dotargs  = list(...))
+                   dotargs  = list(...),
+                   pspace   = pspace)
     class(result) <- c("minconfit", class(result))
     return(result)
   }
@@ -279,13 +281,19 @@ print.minconfit <- function(x, ...) {
             paste(tagvalue, collapse=", "))
     }
   }
-  if(!is.null(fu) && !is.null(da))
-    splat("Fitted by matching theoretical", fu, "function to", da)
-  else {
-    if(!is.null(fu))
+  if(!is.null(fu)) {
+    if(length(fu) > 1) {
+      ## compress names like c("K", "inhom") -> "K[inhom]"
+      fsub <- paste(fu[-1], collapse=",")
+      fu <- paste0(fu[1], paren(fsub, "["))
+    }
+    if(!is.null(da)) {
+      splat("Fitted by matching theoretical", fu, "function to", da)
+    } else {
       splat(" based on", fu)
-    if(!is.null(da))
-      splat(" fitted to", da)
+    }
+  } else if(!is.null(da)) {
+    splat(" fitted to", da)
   }
 
   if(waxlyrical('space', terselevel))
@@ -486,7 +494,7 @@ thomas.estK <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("Thomas")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$K
   
   result <- mincontrast(K, theoret, startpar,
@@ -503,8 +511,8 @@ thomas.estK <- function(X, startpar=c(kappa=1,scale=1),
   ## infer meaningful model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="Thomas")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
@@ -528,11 +536,10 @@ lgcp.estK <- function(X, startpar=c(var=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("LGCP")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
 
   ## digest parameters of Covariance model and test validity
-  ph <- info$parhandler
-  cmodel <- do.call(ph, covmodel)
+  cmodel <- do.call(info$resolveshape, covmodel)$covmodel
   
   theoret <- info$K
 
@@ -554,9 +561,9 @@ lgcp.estK <- function(X, startpar=c(var=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="lgcp")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
-  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
+  result$clustargs <- info$outputshape(cmodel$margs)
   return(result)
 }
 
@@ -579,9 +586,8 @@ matclust.estK <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("MatClust")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$K
-  funaux <-  info$funaux
   
   result <- mincontrast(K, theoret, startpar,
                         ctrl=list(q=q, p=p,rmin=rmin, rmax=rmax),
@@ -590,8 +596,7 @@ matclust.estK <- function(X, startpar=c(kappa=1,scale=1),
                         explain=list(dataname=dataname,
                           fname=attr(K, "fname"),
                           modelname="Matern Cluster process"),
-                        ...,
-                        funaux=funaux)
+                        ...)
   ## imbue with meaning
   par <- result$par
   names(par) <- c("kappa", "R")
@@ -599,8 +604,8 @@ matclust.estK <- function(X, startpar=c(kappa=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="MatClust")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
@@ -626,7 +631,7 @@ thomas.estpcf <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("Thomas")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$pcf
   
   ## avoid using g(0) as it may be infinite
@@ -651,8 +656,8 @@ thomas.estpcf <- function(X, startpar=c(kappa=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="Thomas")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
@@ -676,9 +681,8 @@ matclust.estpcf <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("MatClust")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$pcf
-  funaux <-  info$funaux
   
   ## avoid using g(0) as it may be infinite
   argu <- fvnames(g, ".x")
@@ -693,8 +697,7 @@ matclust.estpcf <- function(X, startpar=c(kappa=1,scale=1),
                         explain=list(dataname=dataname,
                           fname=attr(g, "fname"),
                           modelname="Matern Cluster process"),
-                        ...,
-                        funaux=funaux)
+                        ...)
   ## imbue with meaning
   par <- result$par
   names(par) <- c("kappa", "R")
@@ -702,8 +705,8 @@ matclust.estpcf <- function(X, startpar=c(kappa=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="MatClust")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
@@ -728,11 +731,10 @@ lgcp.estpcf <- function(X, startpar=c(var=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("LGCP")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
 
   ## digest parameters of Covariance model and test validity
-  ph <- info$parhandler
-  cmodel <- do.call(ph, covmodel)
+  cmodel <- do.call(info$resolveshape, covmodel)$covmodel
   
   theoret <- info$pcf
   
@@ -754,9 +756,9 @@ lgcp.estpcf <- function(X, startpar=c(var=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="lgcp")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
-  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
+  result$clustargs <- info$outputshape(cmodel$margs)
   return(result)
 }
 
@@ -784,7 +786,7 @@ cauchy.estK <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("Cauchy")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$K
 
   desc <- "minimum contrast fit of Neyman-Scott process with Cauchy kernel"
@@ -801,8 +803,8 @@ cauchy.estK <- function(X, startpar=c(kappa=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="Cauchy")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
@@ -831,7 +833,7 @@ cauchy.estpcf <- function(X, startpar=c(kappa=1,scale=1),
     stop("Unrecognised format for argument X")
 
   info <- spatstatClusterModelInfo("Cauchy")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$pcf
 
   ## avoid using g(0) as it may be infinite
@@ -855,33 +857,11 @@ cauchy.estpcf <- function(X, startpar=c(kappa=1,scale=1),
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="Cauchy")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
   return(result)
 }
 
-## user-callable
-resolve.vargamma.shape <- function(..., nu.ker=NULL, nu.pcf=NULL, default = FALSE) {
-  if(is.null(nu.ker) && is.null(nu.pcf)){
-    if(!default)
-        stop("Must specify either nu.ker or nu.pcf", call.=FALSE)
-    nu.ker <- -1/4
-  }
-  if(!is.null(nu.ker) && !is.null(nu.pcf))
-    stop("Only one of nu.ker and nu.pcf should be specified",
-         call.=FALSE)
-  if(!is.null(nu.ker)) {
-    check.1.real(nu.ker)
-    stopifnot(nu.ker > -1/2)
-    nu.pcf <- 2 * nu.ker + 1
-  } else {
-    check.1.real(nu.pcf)
-    stopifnot(nu.pcf > 0)
-    nu.ker <- (nu.pcf - 1)/2
-  }
-  return(list(nu.ker=nu.ker, nu.pcf=nu.pcf))
-}
-
 vargamma.estK <- function(X, startpar=c(kappa=1,scale=1), nu = -1/4,
                           lambda=NULL, q=1/4, p=2, rmin=NULL, rmax=NULL,
                           ...) {
@@ -908,20 +888,17 @@ vargamma.estK <- function(X, startpar=c(kappa=1,scale=1), nu = -1/4,
     stop("Unrecognised format for argument X")
 
   ## Catch old nu.ker/nu.pcf syntax and resolve nu-value.
-  dots <- list(...)
-  if(missing(nu)){
-      nu <- resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf, default = TRUE)$nu.ker
-  }
+  if(missing(nu))
+    nu <- resolve.vargamma.shape(..., allow.default = TRUE)$nu.ker
   check.1.real(nu)
   stopifnot(nu > -1/2)
 
   info <- spatstatClusterModelInfo("VarGamma")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$K
   
   ## test validity of parameter nu and digest
-  ph <- info$parhandler
-  cmodel <- ph(nu.ker=nu)
+  cmodel <- info$resolveshape(nu.ker=nu)$covmodel
   margs <- cmodel$margs
 
   desc <- "minimum contrast fit of Neyman-Scott process with Variance Gamma kernel"
@@ -940,9 +917,9 @@ vargamma.estK <- function(X, startpar=c(kappa=1,scale=1), nu = -1/4,
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="VarGamma")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
-  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
+  result$clustargs <- info$outputshape(cmodel$margs)
   return(result)
 }
 
@@ -973,22 +950,17 @@ vargamma.estpcf <- function(X, startpar=c(kappa=1,scale=1), nu=-1/4,
       stop("Unrecognised format for argument X")
   
   ## Catch old nu.ker/nu.pcf syntax and resolve nu-value.
-  dots <- list(...)
-  if(missing(nu)){
-      ## nutmp <- try(resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf)$nu.ker, silent=TRUE)
-      ## if(!inherits(nutmp, "try-error")) nu <- nutmp
-      nu <- resolve.vargamma.shape(nu.ker=dots$nu.ker, nu.pcf=dots$nu.pcf, default = TRUE)$nu.ker
-  }
+  if(missing(nu))
+    nu <- resolve.vargamma.shape(..., allow.default = TRUE)$nu.ker
   check.1.real(nu)
   stopifnot(nu > -1/2)
 
   info <- spatstatClusterModelInfo("VarGamma")
-  startpar <- info$checkpar(startpar)
+  startpar <- info$checkpar(startpar, native=TRUE)
   theoret <- info$pcf
 
   ## test validity of parameter nu and digest 
-  ph <- info$parhandler
-  cmodel <- ph(nu.ker=nu)
+  cmodel <- info$resolveshape(nu.ker=nu)$covmodel
   margs <- cmodel$margs
   
   ## avoid using g(0) as it may be infinite
@@ -1015,9 +987,9 @@ vargamma.estpcf <- function(X, startpar=c(kappa=1,scale=1), nu=-1/4,
   ## infer model parameters
   result$modelpar <- info$interpret(par, lambda)
   result$internal <- list(model="VarGamma")
-  ## add new parametrisation to object
-  result$clustpar <- info$checkpar(par, old=FALSE)
-  result$clustargs <- info$checkclustargs(cmodel$margs, old=FALSE)
+  ## parameters in standard form
+  result$clustpar <- info$checkpar(par, native=FALSE)
+  result$clustargs <- info$outputshape(cmodel$margs)
   return(result)
 }
 
