@@ -1034,12 +1034,16 @@ domain.ppm <- Window.ppm <- function(X, ..., from=c("points", "covariates")) {
   as.owin(X, ..., from=from)
 }
 
+hardcoredist.ppm <- function(x, ..., epsilon=0) {
+  hardcoredist.fii(fitin(x), ..., epsilon=epsilon)
+}
+
 ## change the coefficients in a ppm or other model
 
 tweak.coefs <- function(model, new.coef) {
   if(is.null(new.coef)) return(model)
   co <- coef(model)
-  check.nvector(new.coef, length(co), things="coefficients")
+  check.nvector(new.coef, length(co), things="coefficients", vname="new.coef")
   model$coef.orig <- co
   model$coef <- new.coef
   return(model)
