@@ -3,7 +3,7 @@
 #
 #  Spatial Logistic Regression
 #
-#  $Revision: 1.57 $   $Date: 2021/10/30 05:19:17 $
+#  $Revision: 1.60 $   $Date: 2022/04/06 07:40:12 $
 #
 
 slrm <- function(formula, ..., data=NULL, offset=TRUE, link="logit",
@@ -719,6 +719,10 @@ parameters.slrm <- function(model, ...) { list(trend=coef(model)) }
 simulate.slrm <- function(object, nsim=1, seed=NULL, ...,
                           window=NULL, covariates=NULL, 
                           verbose=TRUE, drop=FALSE) {
+  check.1.integer(nsim)
+  stopifnot(nsim >= 0)
+  if(nsim == 0) return(simulationresult(list()))
+  
   # .... copied from simulate.lm ....
   if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
     runif(1)
@@ -731,7 +735,7 @@ simulate.slrm <- function(object, nsim=1, seed=NULL, ...,
     on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
   }
   starttime <- proc.time()
-  
+
   # determine simulation window and compute intensity
   if(!is.null(window))
     stopifnot(is.owin(window))
@@ -810,8 +814,8 @@ slrmInfluence <- function(model,
     what <- c("leverage", "influence", "dfbetas", "dffit")
   FIT <- model$Fit$FIT
   W <- model$Data$W
-  nr <- nrow(W)
-  nc <- ncol(W)
+  ## nr <- nrow(W)
+  ## nc <- ncol(W)
   result <- list()
   if("leverage" %in% what) {
     h <- hatvalues(FIT, ...)
@@ -873,12 +877,12 @@ emend.slrm <- local({
     }
     # Fitted coefficients
     coef.orig <- coeffs <- coef(object)
-    coefnames  <- names(coeffs)
+    ## coefnames  <- names(coeffs)
     # Trend terms in trend formula
     trendterms <- attr(terms(object), "term.labels")
     # Mapping from coefficients to terms of GLM
     coef2term  <- attr(model.matrix(object), "assign")
-    istrend <- (coef2term > 0)
+    ## istrend <- (coef2term > 0)
     # Identify non-finite trend coefficients
     bad <-  !is.finite(coeffs)
     if(!any(bad)) {
